@@ -1,35 +1,38 @@
-import sys
 import os
 import re
-import threading
+import sys
+import typing as t
 
 import zmq
-from zmq.utils import jsonapi
-
-from tornado.queues import Queue
-from tornado.locks import Event
-
 from IPython.core.getipython import get_ipython
 from IPython.core.inputtransformer2 import leading_empty_lines
+from tornado.locks import Event
+from tornado.queues import Queue
+from zmq.utils import jsonapi
 
 try:
     from jupyter_client.jsonutil import json_default
 except ImportError:
     from jupyter_client.jsonutil import date_default as json_default
 
-from .compiler import (get_file_name, get_tmp_directory, get_tmp_hash_seed)
+from .compiler import get_file_name, get_tmp_directory, get_tmp_hash_seed
 
 try:
     # This import is required to have the next ones working...
     from debugpy.server import api  # noqa
-    from _pydevd_bundle import pydevd_frame_utils
-    from _pydevd_bundle.pydevd_suspended_frames import SuspendedFramesManager, _FramesTracker
+
+    from _pydevd_bundle import pydevd_frame_utils  # isort: skip
+    from _pydevd_bundle.pydevd_suspended_frames import (  # isort: skip
+        SuspendedFramesManager,
+        _FramesTracker,
+    )
+
     _is_debugpy_available = True
 except ImportError:
     _is_debugpy_available = False
 
 # Required for backwards compatiblity
-ROUTING_ID = getattr(zmq, 'ROUTING_ID', None) or zmq.IDENTITY
+ROUTING_ID = getattr(zmq, "ROUTING_ID", None) or zmq.IDENTITY
 
 
 class _FakeCode:
@@ -49,6 +52,7 @@ class _FakeFrame:
 class _DummyPyDB:
     def __init__(self):
         from _pydevd_bundle.pydevd_api import PyDevdAPI
+
         self.variable_presentation = PyDevdAPI.VariablePresentation()
 
 
@@ -61,13 +65,13 @@ class VariableExplorer:
 
     def track(self):
         var = get_ipython().user_ns
-        self.frame = _FakeFrame(_FakeCode('<module>', get_file_name('sys._getframe()')), var, var)
-        self.tracker.track('thread1', pydevd_frame_utils.create_frames_list_from_frame(self.frame))
+        self.frame = _FakeFrame(_FakeCode("<module>", get_file_name("sys._getframe()")), var, var)
+        self.tracker.track("thread1", pydevd_frame_utils.create_frames_list_from_frame(self.frame))
 
     def untrack_all(self):
         self.tracker.untrack_all()
 
-    def get_children_variables(self, variable_ref = None):
+    def get_children_variables(self, variable_ref=None):
         var_ref = variable_ref
         if not var_ref:
             var_ref = id(self.frame)
@@ -77,16 +81,16 @@ class VariableExplorer:
 
 class DebugpyMessageQueue:
 
-    HEADER = 'Content-Length: '
+    HEADER = "Content-Length: "
     HEADER_LENGTH = 16
-    SEPARATOR = '\r\n\r\n'
+    SEPARATOR = "\r\n\r\n"
     SEPARATOR_LENGTH = 4
 
     def __init__(self, event_callback, log):
-        self.tcp_buffer = ''
+        self.tcp_buffer = ""
         self._reset_tcp_pos()
         self.event_callback = event_callback
-        self.message_queue = Queue()
+        self.message_queue: Queue[t.Any] = Queue()
         self.log = log
 
     def _reset_tcp_pos(self):
@@ -96,21 +100,21 @@ class DebugpyMessageQueue:
         self.message_pos = -1
 
     def _put_message(self, raw_msg):
-        self.log.debug('QUEUE - _put_message:')
-        msg = jsonapi.loads(raw_msg)
-        if msg['type'] == 'event':
-            self.log.debug('QUEUE - received event:')
+        self.log.debug("QUEUE - _put_message:")
+        msg = t.cast(t.Dict[str, t.Any], jsonapi.loads(raw_msg))
+        if msg["type"] == "event":
+            self.log.debug("QUEUE - received event:")
             self.log.debug(msg)
             self.event_callback(msg)
         else:
-            self.log.debug('QUEUE - put message:')
+            self.log.debug("QUEUE - put message:")
             self.log.debug(msg)
             self.message_queue.put_nowait(msg)
 
     def put_tcp_frame(self, frame):
         self.tcp_buffer += frame
 
-        self.log.debug('QUEUE - received frame')
+        self.log.debug("QUEUE - received frame")
         while True:
             # Finds header
             if self.header_pos == -1:
@@ -118,37 +122,39 @@ class DebugpyMessageQueue:
             if self.header_pos == -1:
                 return
 
-            self.log.debug('QUEUE - found header at pos %i', self.header_pos)
+            self.log.debug("QUEUE - found header at pos %i", self.header_pos)
 
-            #Finds separator
+            # Finds separator
             if self.separator_pos == -1:
                 hint = self.header_pos + DebugpyMessageQueue.HEADER_LENGTH
                 self.separator_pos = self.tcp_buffer.find(DebugpyMessageQueue.SEPARATOR, hint)
             if self.separator_pos == -1:
                 return
 
-            self.log.debug('QUEUE - found separator at pos %i', self.separator_pos)
+            self.log.debug("QUEUE - found separator at pos %i", self.separator_pos)
 
             if self.message_pos == -1:
                 size_pos = self.header_pos + DebugpyMessageQueue.HEADER_LENGTH
                 self.message_pos = self.separator_pos + DebugpyMessageQueue.SEPARATOR_LENGTH
-                self.message_size = int(self.tcp_buffer[size_pos:self.separator_pos])
+                self.message_size = int(self.tcp_buffer[size_pos : self.separator_pos])
 
-            self.log.debug('QUEUE - found message at pos %i', self.message_pos)
-            self.log.debug('QUEUE - message size is %i', self.message_size)
+            self.log.debug("QUEUE - found message at pos %i", self.message_pos)
+            self.log.debug("QUEUE - message size is %i", self.message_size)
 
             if len(self.tcp_buffer) - self.message_pos < self.message_size:
                 return
 
-            self._put_message(self.tcp_buffer[self.message_pos:self.message_pos + self.message_size])
+            self._put_message(
+                self.tcp_buffer[self.message_pos : self.message_pos + self.message_size]
+            )
             if len(self.tcp_buffer) - self.message_pos == self.message_size:
-                self.log.debug('QUEUE - resetting tcp_buffer')
-                self.tcp_buffer = ''
+                self.log.debug("QUEUE - resetting tcp_buffer")
+                self.tcp_buffer = ""
                 self._reset_tcp_pos()
                 return
             else:
-                self.tcp_buffer = self.tcp_buffer[self.message_pos + self.message_size:]
-                self.log.debug('QUEUE - slicing tcp_buffer: %s', self.tcp_buffer)
+                self.tcp_buffer = self.tcp_buffer[self.message_pos + self.message_size :]
+                self.log.debug("QUEUE - slicing tcp_buffer: %s", self.tcp_buffer)
                 self._reset_tcp_pos()
 
     async def get_message(self):
@@ -156,13 +162,12 @@ class DebugpyMessageQueue:
 
 
 class DebugpyClient:
-
     def __init__(self, log, debugpy_stream, event_callback):
         self.log = log
         self.debugpy_stream = debugpy_stream
         self.event_callback = event_callback
         self.message_queue = DebugpyMessageQueue(self._forward_event, self.log)
-        self.debugpy_host = '127.0.0.1'
+        self.debugpy_host = "127.0.0.1"
         self.debugpy_port = -1
         self.routing_id = None
         self.wait_for_attach = True
@@ -171,12 +176,12 @@ class DebugpyClient:
 
     def _get_endpoint(self):
         host, port = self.get_host_port()
-        return 'tcp://' + host + ':' + str(port)
+        return "tcp://" + host + ":" + str(port)
 
     def _forward_event(self, msg):
-        if msg['event'] == 'initialized':
+        if msg["event"] == "initialized":
             self.init_event.set()
-            self.init_event_seq = msg['seq']
+            self.init_event_seq = msg["seq"]
         self.event_callback(msg)
 
     def _send_request(self, msg):
@@ -189,7 +194,9 @@ class DebugpyClient:
             allow_nan=False,
         )
         content_length = str(len(content))
-        buf = (DebugpyMessageQueue.HEADER + content_length + DebugpyMessageQueue.SEPARATOR).encode('ascii')
+        buf = (DebugpyMessageQueue.HEADER + content_length + DebugpyMessageQueue.SEPARATOR).encode(
+            "ascii"
+        )
         buf += content
         self.log.debug("DEBUGPYCLIENT:")
         self.log.debug(self.routing_id)
@@ -208,9 +215,9 @@ class DebugpyClient:
 
         # 2] Sends configurationDone request
         configurationDone = {
-            'type': 'request',
-            'seq': int(self.init_event_seq) + 1,
-            'command': 'configurationDone'
+            "type": "request",
+            "seq": int(self.init_event_seq) + 1,
+            "command": "configurationDone",
         }
         self._send_request(configurationDone)
 
@@ -224,11 +231,11 @@ class DebugpyClient:
     def get_host_port(self):
         if self.debugpy_port == -1:
             socket = self.debugpy_stream.socket
-            socket.bind_to_random_port('tcp://' + self.debugpy_host)
-            self.endpoint = socket.getsockopt(zmq.LAST_ENDPOINT).decode('utf-8')
+            socket.bind_to_random_port("tcp://" + self.debugpy_host)
+            self.endpoint = socket.getsockopt(zmq.LAST_ENDPOINT).decode("utf-8")
             socket.unbind(self.endpoint)
-            index = self.endpoint.rfind(':')
-            self.debugpy_port = self.endpoint[index+1:]
+            index = self.endpoint.rfind(":")
+            self.debugpy_port = self.endpoint[index + 1 :]
         return self.debugpy_host, self.debugpy_port
 
     def connect_tcp_socket(self):
@@ -247,13 +254,13 @@ class DebugpyClient:
 
     async def send_dap_request(self, msg):
         self._send_request(msg)
-        if self.wait_for_attach and msg['command'] == 'attach':
+        if self.wait_for_attach and msg["command"] == "attach":
             rep = await self._handle_init_sequence()
             self.wait_for_attach = False
             return rep
         else:
             rep = await self._wait_for_response()
-            self.log.debug('DEBUGPYCLIENT - returning:')
+            self.log.debug("DEBUGPYCLIENT - returning:")
             self.log.debug(rep)
             return rep
 
@@ -262,19 +269,21 @@ class Debugger:
 
     # Requests that requires that the debugger has started
     started_debug_msg_types = [
-        'dumpCell', 'setBreakpoints',
-        'source', 'stackTrace',
-        'variables', 'attach',
-        'configurationDone'
+        "dumpCell",
+        "setBreakpoints",
+        "source",
+        "stackTrace",
+        "variables",
+        "attach",
+        "configurationDone",
     ]
 
     # Requests that can be handled even if the debugger is not running
-    static_debug_msg_types = [
-        'debugInfo', 'inspectVariables', 
-        'richInspectVariables', 'modules'
-    ]
+    static_debug_msg_types = ["debugInfo", "inspectVariables", "richInspectVariables", "modules"]
 
-    def __init__(self, log, debugpy_stream, event_callback, shell_socket, session, just_my_code = True):
+    def __init__(
+        self, log, debugpy_stream, event_callback, shell_socket, session, just_my_code=True
+    ):
         self.log = log
         self.debugpy_client = DebugpyClient(log, debugpy_stream, self._handle_event)
         self.shell_socket = shell_socket
@@ -282,7 +291,7 @@ class Debugger:
         self.is_started = False
         self.event_callback = event_callback
         self.just_my_code = just_my_code
-        self.stopped_queue = Queue()
+        self.stopped_queue: Queue[t.Any] = Queue()
 
         self.started_debug_handlers = {}
         for msg_type in Debugger.started_debug_msg_types:
@@ -298,26 +307,26 @@ class Debugger:
         self.debugpy_initialized = False
         self._removed_cleanup = {}
 
-        self.debugpy_host = '127.0.0.1'
+        self.debugpy_host = "127.0.0.1"
         self.debugpy_port = 0
         self.endpoint = None
 
         self.variable_explorer = VariableExplorer()
 
     def _handle_event(self, msg):
-        if msg['event'] == 'stopped':
-            if msg['body']['allThreadsStopped']:
+        if msg["event"] == "stopped":
+            if msg["body"]["allThreadsStopped"]:
                 self.stopped_queue.put_nowait(msg)
                 # Do not forward the event now, will be done in the handle_stopped_event
                 return
             else:
-                self.stopped_threads.add(msg['body']['threadId'])
+                self.stopped_threads.add(msg["body"]["threadId"])
                 self.event_callback(msg)
-        elif msg['event'] == 'continued':
-            if msg['body']['allThreadsContinued']:
+        elif msg["event"] == "continued":
+            if msg["body"]["allThreadsContinued"]:
                 self.stopped_threads = set()
             else:
-                self.stopped_threads.remove(msg['body']['threadId'])
+                self.stopped_threads.remove(msg["body"]["threadId"])
             self.event_callback(msg)
         else:
             self.event_callback(msg)
@@ -326,43 +335,32 @@ class Debugger:
         return await self.debugpy_client.send_dap_request(msg)
 
     def _build_variables_response(self, request, variables):
-        var_list = [var for var in variables if self.accept_variable(var['name'])]
+        var_list = [var for var in variables if self.accept_variable(var["name"])]
         reply = {
-            'seq': request['seq'],
-            'type': 'response',
-            'request_seq': request['seq'],
-            'success': True,
-            'command': request['command'],
-            'body': {
-                'variables': var_list
-            }
+            "seq": request["seq"],
+            "type": "response",
+            "request_seq": request["seq"],
+            "success": True,
+            "command": request["command"],
+            "body": {"variables": var_list},
         }
         return reply
 
     def _accept_stopped_thread(self, thread_name):
         # TODO: identify Thread-2, Thread-3 and Thread-4. These are NOT
         # Control, IOPub or Heartbeat threads
-        forbid_list = [
-            'IPythonHistorySavingThread',
-            'Thread-2',
-            'Thread-3',
-            'Thread-4'
-        ]
+        forbid_list = ["IPythonHistorySavingThread", "Thread-2", "Thread-3", "Thread-4"]
         return thread_name not in forbid_list
-        
+
     async def handle_stopped_event(self):
         # Wait for a stopped event message in the stopped queue
         # This message is used for triggering the 'threads' request
         event = await self.stopped_queue.get()
-        req = {
-            'seq': event['seq'] + 1,
-            'type': 'request',
-            'command': 'threads'
-        }
+        req = {"seq": event["seq"] + 1, "type": "request", "command": "threads"}
         rep = await self._forward_message(req)
-        for t in rep['body']['threads']:
-            if self._accept_stopped_thread(t['name']):
-                self.stopped_threads.add(t['id'])
+        for thread in rep["body"]["threads"]:
+            if self._accept_stopped_thread(thread["name"]):
+                self.stopped_threads.add(thread["id"])
         self.event_callback(event)
 
     @property
@@ -375,17 +373,19 @@ class Debugger:
             if not os.path.exists(tmp_dir):
                 os.makedirs(tmp_dir)
             host, port = self.debugpy_client.get_host_port()
-            code = 'import debugpy;'
-            code += 'debugpy.listen(("' + host + '",' + port + '))'
-            content = {
-                'code': code,
-                'silent': True
-            }
-            self.session.send(self.shell_socket, 'execute_request', content,
-                              None, (self.shell_socket.getsockopt(ROUTING_ID)))
+            code = "import debugpy;"
+            code += 'debugpy.listen(("' + host + '",' + port + "))"
+            content = {"code": code, "silent": True}
+            self.session.send(
+                self.shell_socket,
+                "execute_request",
+                content,
+                None,
+                (self.shell_socket.getsockopt(ROUTING_ID)),
+            )
 
             ident, msg = self.session.recv(self.shell_socket, mode=0)
-            self.debugpy_initialized = msg['content']['status'] == 'ok'
+            self.debugpy_initialized = msg["content"]["status"] == "ok"
 
         # Don't remove leading empty lines when debugging so the breakpoints are correctly positioned
         cleanup_transforms = get_ipython().input_transformer_manager.cleanup_transforms
@@ -406,20 +406,18 @@ class Debugger:
             cleanup_transforms.insert(index, func)
 
     async def dumpCell(self, message):
-        code = message['arguments']['code']
+        code = message["arguments"]["code"]
         file_name = get_file_name(code)
 
-        with open(file_name, 'w', encoding='utf-8') as f:
+        with open(file_name, "w", encoding="utf-8") as f:
             f.write(code)
 
         reply = {
-            'type': 'response',
-            'request_seq': message['seq'],
-            'success': True,
-            'command': message['command'],
-            'body': {
-                'sourcePath': file_name
-            }
+            "type": "response",
+            "request_seq": message["seq"],
+            "success": True,
+            "command": message["command"],
+            "body": {"sourcePath": file_name},
         }
         return reply
 
@@ -429,22 +427,16 @@ class Debugger:
         return await self._forward_message(message)
 
     async def source(self, message):
-        reply = {
-            'type': 'response',
-            'request_seq': message['seq'],
-            'command': message['command']
-        }
+        reply = {"type": "response", "request_seq": message["seq"], "command": message["command"]}
         source_path = message["arguments"]["source"]["path"]
         if os.path.isfile(source_path):
-            with open(source_path, encoding='utf-8') as f:
-                reply['success'] = True
-                reply['body'] = {
-                    'content': f.read()
-                }
+            with open(source_path, encoding="utf-8") as f:
+                reply["success"] = True
+                reply["body"] = {"content": f.read()}
         else:
-            reply['success'] = False
-            reply['message'] = 'source unavailable'
-            reply['body'] = {}
+            reply["success"] = False
+            reply["message"] = "source unavailable"
+            reply["body"] = {}
 
         return reply
 
@@ -462,105 +454,98 @@ class Debugger:
         try:
             sf_list = reply["body"]["stackFrames"]
             module_idx = len(sf_list) - next(
-                i
-                for i, v in enumerate(reversed(sf_list), 1)
-                if v["name"] == "<module>" and i != 1
+                i for i, v in enumerate(reversed(sf_list), 1) if v["name"] == "<module>" and i != 1
             )
-            reply["body"]["stackFrames"] = reply["body"]["stackFrames"][
-                : module_idx + 1
-            ]
+            reply["body"]["stackFrames"] = reply["body"]["stackFrames"][: module_idx + 1]
         except StopIteration:
             pass
         return reply
 
     def accept_variable(self, variable_name):
         forbid_list = [
-            '__name__',
-            '__doc__',
-            '__package__',
-            '__loader__',
-            '__spec__',
-            '__annotations__',
-            '__builtins__',
-            '__builtin__',
-            '__display__',
-            'get_ipython',
-            'debugpy',
-            'exit',
-            'quit',
-            'In',
-            'Out',
-            '_oh',
-            '_dh',
-            '_',
-            '__',
-            '___'
+            "__name__",
+            "__doc__",
+            "__package__",
+            "__loader__",
+            "__spec__",
+            "__annotations__",
+            "__builtins__",
+            "__builtin__",
+            "__display__",
+            "get_ipython",
+            "debugpy",
+            "exit",
+            "quit",
+            "In",
+            "Out",
+            "_oh",
+            "_dh",
+            "_",
+            "__",
+            "___",
         ]
         cond = variable_name not in forbid_list
-        cond = cond and not bool(re.search(r'^_\d', variable_name))
-        cond = cond and variable_name[0:2] != '_i'
+        cond = cond and not bool(re.search(r"^_\d", variable_name))
+        cond = cond and variable_name[0:2] != "_i"
         return cond
 
     async def variables(self, message):
         reply = {}
         if not self.stopped_threads:
-            variables = self.variable_explorer.get_children_variables(message['arguments']['variablesReference'])
+            variables = self.variable_explorer.get_children_variables(
+                message["arguments"]["variablesReference"]
+            )
             return self._build_variables_response(message, variables)
         else:
             reply = await self._forward_message(message)
             # TODO : check start and count arguments work as expected in debugpy
-            reply['body']['variables'] = \
-                [var for var in reply['body']['variables'] if self.accept_variable(var['name'])]
+            reply["body"]["variables"] = [
+                var for var in reply["body"]["variables"] if self.accept_variable(var["name"])
+            ]
         return reply
 
     async def attach(self, message):
         host, port = self.debugpy_client.get_host_port()
-        message['arguments']['connect'] = {
-            'host': host,
-            'port': port
-        }
-        message['arguments']['logToFile'] = True
+        message["arguments"]["connect"] = {"host": host, "port": port}
+        message["arguments"]["logToFile"] = True
         # Experimental option to break in non-user code.
         # The ipykernel source is in the call stack, so the user
         # has to manipulate the step-over and step-into in a wize way.
         # Set debugOptions for breakpoints in python standard library source.
         if not self.just_my_code:
-            message['arguments']['debugOptions'] = [ 'DebugStdLib' ]
+            message["arguments"]["debugOptions"] = ["DebugStdLib"]
         return await self._forward_message(message)
 
     async def configurationDone(self, message):
         reply = {
-            'seq': message['seq'],
-            'type': 'response',
-            'request_seq': message['seq'],
-            'success': True,
-            'command': message['command']
+            "seq": message["seq"],
+            "type": "response",
+            "request_seq": message["seq"],
+            "success": True,
+            "command": message["command"],
         }
         return reply
 
     async def debugInfo(self, message):
         breakpoint_list = []
         for key, value in self.breakpoint_list.items():
-            breakpoint_list.append({
-                'source': key,
-                'breakpoints': value
-            })
+            breakpoint_list.append({"source": key, "breakpoints": value})
         reply = {
-            'type': 'response',
-            'request_seq': message['seq'],
-            'success': True,
-            'command': message['command'],
-            'body': {
-                'isStarted': self.is_started,
-                'hashMethod': 'Murmur2',
-                'hashSeed': get_tmp_hash_seed(),
-                'tmpFilePrefix': get_tmp_directory() + os.sep,
-                'tmpFileSuffix': '.py',
-                'breakpoints': breakpoint_list,
-                'stoppedThreads': list(self.stopped_threads),
-                'richRendering': True,
-                'exceptionPaths': ['Python Exceptions']
-            }
+            "type": "response",
+            "request_seq": message["seq"],
+            "success": True,
+            "command": message["command"],
+            "body": {
+                "isStarted": self.is_started,
+                "hashMethod": "Murmur2",
+                "hashSeed": get_tmp_hash_seed(),
+                "tmpFilePrefix": get_tmp_directory() + os.sep,
+                "tmpFileSuffix": ".py",
+                "breakpoints": breakpoint_list,
+                "stoppedThreads": list(self.stopped_threads),
+                "richRendering": True,
+                "exceptionPaths": ["Python Exceptions"],
+            },
         }
         return reply
 
@@ -627,56 +612,52 @@ class Debugger:
 
     async def modules(self, message):
         modules = list(sys.modules.values())
-        startModule = message.get('startModule', 0)
-        moduleCount = message.get('moduleCount', len(modules))
+        startModule = message.get("startModule", 0)
+        moduleCount = message.get("moduleCount", len(modules))
         mods = []
         for i in range(startModule, moduleCount):
             module = modules[i]
-            filename = getattr(getattr(module, '__spec__', None), 'origin', None)
-            if filename and filename.endswith('.py'):
-                mods.append({
-                    'id': i,
-                    'name': module.__name__,
-                    'path': filename
-                })
+            filename = getattr(getattr(module, "__spec__", None), "origin", None)
+            if filename and filename.endswith(".py"):
+                mods.append({"id": i, "name": module.__name__, "path": filename})
 
-        reply = {'body': {'modules': mods, 'totalModules': len(modules)}}
+        reply = {"body": {"modules": mods, "totalModules": len(modules)}}
         return reply
 
     async def process_request(self, message):
         reply = {}
 
-        if message['command'] == 'initialize':
+        if message["command"] == "initialize":
             if self.is_started:
-                self.log.info('The debugger has already started')
+                self.log.info("The debugger has already started")
             else:
                 self.is_started = self.start()
                 if self.is_started:
-                    self.log.info('The debugger has started')
+                    self.log.info("The debugger has started")
                 else:
                     reply = {
-                        'command': 'initialize',
-                        'request_seq': message['seq'],
-                        'seq': 3,
-                        'success': False,
-                        'type': 'response'
+                        "command": "initialize",
+                        "request_seq": message["seq"],
+                        "seq": 3,
+                        "success": False,
+                        "type": "response",
                     }
 
-        handler = self.static_debug_handlers.get(message['command'], None)
+        handler = self.static_debug_handlers.get(message["command"], None)
         if handler is not None:
             reply = await handler(message)
         elif self.is_started:
-            handler = self.started_debug_handlers.get(message['command'], None)
+            handler = self.started_debug_handlers.get(message["command"], None)
             if handler is not None:
                 reply = await handler(message)
             else:
                 reply = await self._forward_message(message)
 
-        if message['command'] == 'disconnect':
+        if message["command"] == "disconnect":
             self.stop()
             self.breakpoint_list = {}
             self.stopped_threads = set()
             self.is_started = False
-            self.log.info('The debugger has stopped')
+            self.log.info("The debugger has stopped")
 
         return reply
