@@ -4,42 +4,42 @@
 # Distributed under the terms of the Modified BSD License.
 
 import atexit
-from binascii import b2a_hex
-from collections import deque
-from imp import lock_held as import_lock_held
+import io
 import os
 import sys
 import threading
-import warnings
-from weakref import WeakSet
 import traceback
+import warnings
+from binascii import b2a_hex
+from collections import deque
 from io import StringIO, TextIOBase
-import io
+from typing import Any, Callable, Optional
+from weakref import WeakSet
 
 import zmq
+
 if zmq.pyzmq_version_info() >= (17, 0):
     from tornado.ioloop import IOLoop
 else:
     # deprecated since pyzmq 17
     from zmq.eventloop.ioloop import IOLoop
-from zmq.eventloop.zmqstream import ZMQStream
 
 from jupyter_client.session import extract_header
+from zmq.eventloop.zmqstream import ZMQStream
 
-
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Globals
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 MASTER = 0
 CHILD = 1
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # IO classes
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 
-class IOPubThread(object):
+class IOPubThread:
     """An object for sending IOPub messages in a background thread
 
     Prevents a blocking main thread from delaying output from threads.
@@ -67,13 +67,14 @@ class IOPubThread(object):
         if pipe:
             self._setup_pipe_in()
         self._local = threading.local()
-        self._events = deque()
-        self._event_pipes = WeakSet()
+        self._events: deque[Callable[..., Any]] = deque()
+        self._event_pipes: WeakSet[Any] = WeakSet()
         self._setup_event_pipe()
-        self.thread = threading.Thread(target=self._thread_main)
+        self.thread = threading.Thread(target=self._thread_main, name="IOPub")
         self.thread.daemon = True
-        self.thread.pydev_do_not_trace = True
-        self.thread.is_pydev_daemon_thread = True
+        self.thread.pydev_do_not_trace = True  # type:ignore[attr-defined]
+        self.thread.is_pydev_daemon_thread = True  # type:ignore[attr-defined]
+        self.thread.name = "IOPub"
 
     def _thread_main(self):
         """The inner loop that's actually run in a thread"""
@@ -87,8 +88,8 @@ class IOPubThread(object):
         pipe_in = ctx.socket(zmq.PULL)
         pipe_in.linger = 0
 
-        _uuid = b2a_hex(os.urandom(16)).decode('ascii')
-        iface = self._event_interface = 'inproc://%s' % _uuid
+        _uuid = b2a_hex(os.urandom(16)).decode("ascii")
+        iface = self._event_interface = "inproc://%s" % _uuid
         pipe_in.bind(iface)
         self._event_puller = ZMQStream(pipe_in, self.io_loop)
         self._event_puller.on_recv(self._handle_event)
@@ -121,7 +122,7 @@ class IOPubThread(object):
         # freeze event count so new writes don't extend the queue
         # while we are processing
         n_events = len(self._events)
-        for i in range(n_events):
+        for _ in range(n_events):
             event_f = self._events.popleft()
             event_f()
 
@@ -138,8 +139,9 @@ class IOPubThread(object):
         try:
             self._pipe_port = pipe_in.bind_to_random_port("tcp://127.0.0.1")
         except zmq.ZMQError as e:
-            warnings.warn("Couldn't bind IOPub Pipe to 127.0.0.1: %s" % e +
-                "\nsubprocess output will be unavailable."
+            warnings.warn(
+                "Couldn't bind IOPub Pipe to 127.0.0.1: %s" % e
+                + "\nsubprocess output will be unavailable."
             )
             self._pipe_flag = False
             pipe_in.close()
@@ -160,7 +162,7 @@ class IOPubThread(object):
         # must be new context after fork
         ctx = zmq.Context()
         pipe_out = ctx.socket(zmq.PUSH)
-        pipe_out.linger = 3000 # 3s timeout for pipe_out sends before discarding the message
+        pipe_out.linger = 3000  # 3s timeout for pipe_out sends before discarding the message
         pipe_out.connect("tcp://127.0.0.1:%i" % self._pipe_port)
         return ctx, pipe_out
 
@@ -176,6 +178,7 @@ class IOPubThread(object):
 
     def start(self):
         """Start the IOPub thread"""
+        self.thread.name = "IOPub"
         self.thread.start()
         # make sure we don't prevent process exit
         # I'm not sure why setting daemon=True above isn't enough, but it doesn't appear to be.
@@ -211,7 +214,7 @@ class IOPubThread(object):
         if self.thread.is_alive():
             self._events.append(f)
             # wake event thread (message content is ignored)
-            self._event_pipe.send(b'')
+            self._event_pipe.send(b"")
         else:
             f()
 
@@ -220,10 +223,13 @@ class IOPubThread(object):
 
         If my thread isn't running (e.g. forked process), send immediately.
         """
-        self.schedule(lambda : self._really_send(*args, **kwargs))
+        self.schedule(lambda: self._really_send(*args, **kwargs))
 
     def _really_send(self, msg, *args, **kwargs):
         """The callback that actually sends messages"""
+        if self.closed:
+            return
+
         mp_mode = self._check_mp_mode()
 
         if mp_mode != CHILD:
@@ -239,8 +245,9 @@ class IOPubThread(object):
             ctx.term()
 
 
-class BackgroundSocket(object):
+class BackgroundSocket:
     """Wrapper around IOPub thread that provides zmq send[_multipart]"""
+
     io_thread = None
 
     def __init__(self, io_thread):
@@ -248,21 +255,33 @@ class BackgroundSocket(object):
 
     def __getattr__(self, attr):
         """Wrap socket attr access for backward-compatibility"""
-        if attr.startswith('__') and attr.endswith('__'):
+        if attr.startswith("__") and attr.endswith("__"):
             # don't wrap magic methods
-            super(BackgroundSocket, self).__getattr__(attr)
+            super().__getattr__(attr)  # type:ignore[misc]
+        assert self.io_thread is not None
         if hasattr(self.io_thread.socket, attr):
-            warnings.warn("Accessing zmq Socket attribute %s on BackgroundSocket" % attr,
-                DeprecationWarning, stacklevel=2)
+            warnings.warn(
+                f"Accessing zmq Socket attribute {attr} on BackgroundSocket"
+                f" is deprecated since ipykernel 4.3.0"
+                f" use .io_thread.socket.{attr}",
+                DeprecationWarning,
+                stacklevel=2,
+            )
             return getattr(self.io_thread.socket, attr)
-        super(BackgroundSocket, self).__getattr__(attr)
+        super().__getattr__(attr)  # type:ignore[misc]
 
     def __setattr__(self, attr, value):
-        if attr == 'io_thread' or (attr.startswith('__' and attr.endswith('__'))):
-            super(BackgroundSocket, self).__setattr__(attr, value)
+        if attr == "io_thread" or (attr.startswith("__" and attr.endswith("__"))):
+            super().__setattr__(attr, value)
         else:
-            warnings.warn("Setting zmq Socket attribute %s on BackgroundSocket" % attr,
-                DeprecationWarning, stacklevel=2)
+            warnings.warn(
+                f"Setting zmq Socket attribute {attr} on BackgroundSocket"
+                f" is deprecated since ipykernel 4.3.0"
+                f" use .io_thread.socket.{attr}",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            assert self.io_thread is not None
             setattr(self.io_thread.socket, attr, value)
 
     def send(self, msg, *args, **kwargs):
@@ -270,6 +289,7 @@ class BackgroundSocket(object):
 
     def send_multipart(self, *args, **kwargs):
         """Schedule send in IO thread"""
+        assert self.io_thread is not None
         return self.io_thread.send_multipart(*args, **kwargs)
 
 
@@ -285,8 +305,8 @@ class OutStream(TextIOBase):
     # The time interval between automatic flushes, in seconds.
     flush_interval = 0.2
     topic = None
-    encoding = 'UTF-8'
-
+    encoding = "UTF-8"
+    _exc: Optional[Any] = None
 
     def fileno(self):
         """
@@ -320,7 +340,15 @@ class OutStream(TextIOBase):
             self._exc = sys.exc_info()
 
     def __init__(
-        self, session, pub_thread, name, pipe=None, echo=None, *, watchfd=True, isatty=False,
+        self,
+        session,
+        pub_thread,
+        name,
+        pipe=None,
+        echo=None,
+        *,
+        watchfd=True,
+        isatty=False,
     ):
         """
         Parameters
@@ -339,8 +367,7 @@ class OutStream(TextIOBase):
         """
         if pipe is not None:
             warnings.warn(
-                "pipe argument to OutStream is deprecated and ignored",
-                " since ipykernel 4.2.3.",
+                "pipe argument to OutStream is deprecated and ignored since ipykernel 4.2.3.",
                 DeprecationWarning,
                 stacklevel=2,
             )
@@ -364,7 +391,8 @@ class OutStream(TextIOBase):
         self._flush_pending = False
         self._subprocess_flush_pending = False
         self._io_loop = pub_thread.io_loop
-        self._new_buffer()
+        self._buffer_lock = threading.RLock()
+        self._buffer = StringIO()
         self.echo = None
         self._isatty = bool(isatty)
 
@@ -379,7 +407,7 @@ class OutStream(TextIOBase):
             self._setup_stream_redirects(name)
 
         if echo:
-            if hasattr(echo, 'read') and hasattr(echo, 'write'):
+            if hasattr(echo, "read") and hasattr(echo, "write"):
                 self.echo = echo
             else:
                 raise ValueError("echo argument must be a file like object")
@@ -436,6 +464,7 @@ class OutStream(TextIOBase):
         # add_timeout has to be handed to the io thread via event pipe
         def _schedule_in_thread():
             self._io_loop.call_later(self.flush_interval, self._flush)
+
         self.pub_thread.schedule(_schedule_in_thread)
 
     def flush(self):
@@ -443,20 +472,22 @@ class OutStream(TextIOBase):
 
         send will happen in the background thread
         """
-        if self.pub_thread and self.pub_thread.thread is not None and self.pub_thread.thread.is_alive():
+        if (
+            self.pub_thread
+            and self.pub_thread.thread is not None
+            and self.pub_thread.thread.is_alive()
+            and self.pub_thread.thread.ident != threading.current_thread().ident
+        ):
             # request flush on the background thread
             self.pub_thread.schedule(self._flush)
             # wait for flush to actually get through, if we can.
-            # waiting across threads during import can cause deadlocks
-            # so only wait if import lock is not held
-            if not import_lock_held():
-                evt = threading.Event()
-                self.pub_thread.schedule(evt.set)
-                # and give a timeout to avoid
-                if not evt.wait(self.flush_timeout):
-                    # write directly to __stderr__ instead of warning because
-                    # if this is happening sys.stderr may be the problem.
-                    print("IOStream.flush timed out", file=sys.__stderr__)
+            evt = threading.Event()
+            self.pub_thread.schedule(evt.set)
+            # and give a timeout to avoid
+            if not evt.wait(self.flush_timeout):
+                # write directly to __stderr__ instead of warning because
+                # if this is happening sys.stderr may be the problem.
+                print("IOStream.flush timed out", file=sys.__stderr__)
         else:
             self._flush()
 
@@ -474,8 +505,7 @@ class OutStream(TextIOBase):
                 self.echo.flush()
             except OSError as e:
                 if self.echo is not sys.__stderr__:
-                    print("Flush failed: {}".format(e),
-                          file=sys.__stderr__)
+                    print(f"Flush failed: {e}", file=sys.__stderr__)
 
         data = self._flush_buffer()
         if data:
@@ -483,11 +513,16 @@ class OutStream(TextIOBase):
             # since pub_thread is itself fork-safe.
             # There should be a better way to do this.
             self.session.pid = os.getpid()
-            content = {'name':self.name, 'text':data}
-            self.session.send(self.pub_thread, 'stream', content=content,
-                parent=self.parent_header, ident=self.topic)
+            content = {"name": self.name, "text": data}
+            self.session.send(
+                self.pub_thread,
+                "stream",
+                content=content,
+                parent=self.parent_header,
+                ident=self.topic,
+            )
 
-    def write(self, string: str) -> int:
+    def write(self, string: str) -> Optional[int]:  # type:ignore[override]
         """Write to current stream after encoding if necessary
 
         Returns
@@ -498,30 +533,28 @@ class OutStream(TextIOBase):
         """
 
         if not isinstance(string, str):
-            raise ValueError(
-                "TypeError: write() argument must be str, not {type(string)}"
-            )
+            raise TypeError(f"write() argument must be str, not {type(string)}")
 
         if self.echo is not None:
             try:
                 self.echo.write(string)
             except OSError as e:
                 if self.echo is not sys.__stderr__:
-                    print("Write failed: {}".format(e),
-                          file=sys.__stderr__)
+                    print(f"Write failed: {e}", file=sys.__stderr__)
 
         if self.pub_thread is None:
-            raise ValueError('I/O operation on closed file')
+            raise ValueError("I/O operation on closed file")
         else:
 
-            is_child = (not self._is_master_process())
+            is_child = not self._is_master_process()
             # only touch the buffer in the IO thread to avoid races
-            self.pub_thread.schedule(lambda: self._buffer.write(string))
+            with self._buffer_lock:
+                self._buffer.write(string)
             if is_child:
                 # mp.Pool cannot be trusted to flush promptly (or ever),
                 # and this helps.
                 if self._subprocess_flush_pending:
-                    return
+                    return None
                 self._subprocess_flush_pending = True
                 # We can not rely on self._io_loop.call_later from a subprocess
                 self.pub_thread.schedule(self._flush)
@@ -532,7 +565,7 @@ class OutStream(TextIOBase):
 
     def writelines(self, sequence):
         if self.pub_thread is None:
-            raise ValueError('I/O operation on closed file')
+            raise ValueError("I/O operation on closed file")
         else:
             for string in sequence:
                 self.write(string)
@@ -541,17 +574,15 @@ class OutStream(TextIOBase):
         return True
 
     def _flush_buffer(self):
-        """clear the current buffer and return the current buffer data.
-
-        This should only be called in the IO thread.
-        """
-        data = ''
-        if self._buffer is not None:
-            buf = self._buffer
-            self._new_buffer()
-            data = buf.getvalue()
-            buf.close()
+        """clear the current buffer and return the current buffer data."""
+        buf = self._rotate_buffer()
+        data = buf.getvalue()
+        buf.close()
         return data
 
-    def _new_buffer(self):
-        self._buffer = StringIO()
+    def _rotate_buffer(self):
+        """Returns the current buffer and replaces it with an empty buffer."""
+        with self._buffer_lock:
+            old_buffer = self._buffer
+            self._buffer = StringIO()
+        return old_buffer
